#include <mpi.h>

extern MPI_Request* req;
extern MPI_Status* status;

void jacobi_iteration_nb (int my_rank, int size, int n_i,
			  double* rhs, double* x_k1, double* x_k)
{
  int i;

  /* calculate values on the end-points */
  x_k[1] = 0.5*(rhs[1]+x_k1[0]+x_k1[2]);
  x_k[n_i] = 0.5*(rhs[n_i]+x_k1[n_i-1]+x_k1[n_i+1]);

  if (my_rank>0) {
    /* receive from left neighbor */
    MPI_Irecv (&(x_k[0]),1,MPI_DOUBLE,
	      my_rank-1,501,MPI_COMM_WORLD,&req[0]);
    /* send left neighbor */
    MPI_Isend (&(x_k[1]),1,MPI_DOUBLE,
	      my_rank-1,502,MPI_COMM_WORLD,&req[1]);
  }

  if (my_rank<size-1) {
    /* send to right neighbor */
    MPI_Isend (&(x_k[n_i]),1,MPI_DOUBLE,
	      my_rank+1,501,MPI_COMM_WORLD,&req[2]);
    /* receive from right neighbor */
    MPI_Irecv (&(x_k[n_i+1]),1,MPI_DOUBLE,
	      my_rank+1,502,MPI_COMM_WORLD,&req[3]);
  }

  /* calculate values on the inner-points */
  for (i=2; i<n_i; i++)
    x_k[i] = 0.5*(rhs[i]+x_k1[i-1]+x_k1[i+1]);

  if (my_rank>0) {
    MPI_Wait (&req[0],&status[0]);
    MPI_Wait (&req[1],&status[1]);
  }

  if (my_rank<size-1) {
    MPI_Wait (&req[2],&status[2]);
    MPI_Wait (&req[3],&status[3]);
  }
}
